@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -21,6 +21,9 @@
     limitations under the License.
 
 ===============================================================================
+
+**Dictionary manipulations.**
+
 """
 
 from typing import Any, Callable, Dict, Hashable, List, Optional
@@ -31,6 +34,10 @@ from typing import Any, Callable, Dict, Hashable, List, Optional
 # =============================================================================
 
 def get_case_insensitive_dict_key(d: Dict, k: str) -> Optional[str]:
+    """
+    Within the dictionary ``d``, find a key that matches (in case-insensitive
+    fashion) the key ``k``, and return it (or ``None`` if there isn't one).
+    """
     for key in d.keys():
         if k.lower() == key.lower():
             return key
@@ -39,10 +46,11 @@ def get_case_insensitive_dict_key(d: Dict, k: str) -> Optional[str]:
 
 def merge_dicts(*dict_args: Dict) -> Dict:
     """
-    Given any number of dicts, shallow copy and merge into a new dict,
-    precedence goes to key value pairs in latter dicts.
+    Given any number of dicts, shallow-copy them and merge into a new dict.
+    Precedence goes to key/value pairs in dicts that are later in the list.
+
+    See http://stackoverflow.com/questions/38987.
     """
-    # http://stackoverflow.com/questions/38987
     result = {}
     for dictionary in dict_args:
         result.update(dictionary)
@@ -63,22 +71,34 @@ def merge_two_dicts(x: Dict, y: Dict) -> Dict:
 
         z = {**x, **y}
 
-    See http://stackoverflow.com/questions/38987
+    See http://stackoverflow.com/questions/38987.
     """
     z = x.copy()
     z.update(y)
     return z
 
 
-def rename_key(kwargs: Dict[str, Any], old: str, new: str) -> None:
-    kwargs[new] = kwargs.pop(old)
+def rename_key(d: Dict[str, Any], old: str, new: str) -> None:
+    """
+    Rename a key in dictionary ``d`` from ``old`` to ``new``, in place.
+    """
+    d[new] = d.pop(old)
 
 
 def rename_keys(d: Dict[str, Any], mapping: Dict[str, str]) -> Dict[str, Any]:
     """
-    Renames keys in a dictionary according to the mapping to -> from.
-    Leave other keys unchanged.
-    Does NOT modify the original dictionaries.
+    Returns a copy of the dictionary ``d`` with its keys renamed according to
+    ``mapping``.
+
+    Args:
+        d: the starting dictionary
+        mapping: a dictionary of the format ``{old_key_name: new_key_name}``
+
+    Returns:
+        a new dictionary
+
+    Keys that are not in ``mapping`` are left unchanged.
+    The input parameters are not modified.
     """
     result = {}  # type: Dict[str, Any]
     for k, v in d.items():
@@ -90,9 +110,16 @@ def rename_keys(d: Dict[str, Any], mapping: Dict[str, str]) -> Dict[str, Any]:
 
 def rename_keys_in_dict(d: Dict[str, Any], renames: Dict[str, str]) -> None:
     """
-    Renames, IN PLACE, the keys in d according to the mapping in "renames".
-    """
-    # https://stackoverflow.com/questions/4406501/change-the-name-of-a-key-in-dictionary  # noqa
+    Renames, IN PLACE, the keys in ``d`` according to the mapping in
+    ``renames``.
+    
+    Args:
+        d: a dictionary to modify 
+        renames: a dictionary of the format ``{old_key_name: new_key_name}``
+        
+    See
+    https://stackoverflow.com/questions/4406501/change-the-name-of-a-key-in-dictionary.
+    """  # noqa
     for old_key, new_key in renames.items():
         if new_key == old_key:
             continue
@@ -106,8 +133,8 @@ def rename_keys_in_dict(d: Dict[str, Any], renames: Dict[str, str]) -> None:
 
 def prefix_dict_keys(d: Dict[str, Any], prefix: str) -> Dict[str, Any]:
     """
-    Returns a dictionary that's the same as d but with prefix prepended to its
-    keys.
+    Returns a dictionary that's a copy of as ``d`` but with ``prefix``
+    prepended to its keys.
     """
     result = {}  # type: Dict[str, Any]
     for k, v in d.items():
@@ -117,7 +144,7 @@ def prefix_dict_keys(d: Dict[str, Any], prefix: str) -> Dict[str, Any]:
 
 def reversedict(d: Dict[Any, Any]) -> Dict[Any, Any]:
     """
-    Takes a k -> v mapping and returns a v -> k mapping.
+    Takes a ``k -> v`` mapping and returns a ``v -> k`` mapping.
     """
     return {v: k for k, v in d.items()}
 
@@ -125,7 +152,8 @@ def reversedict(d: Dict[Any, Any]) -> Dict[Any, Any]:
 def set_null_values_in_dict(d: Dict[str, Any],
                             null_literals: List[Any]) -> None:
     """
-    Within d (in place), replace any values found in null_literals with None.
+    Within ``d`` (in place), replace any values found in ``null_literals`` with
+    ``None``.
     """
     if not null_literals:
         return
@@ -143,11 +171,12 @@ def map_keys_to_values(l: List[Any], d: Dict[Any, Any], default: Any = None,
                        raise_if_missing: bool = False,
                        omit_if_missing: bool = False) -> List[Any]:
     """
-    The "d" dictionary contains a key -> value mapping.
-    We start with a list of potential keys in "l", and return a list of
-    corresponding values -- substituting "default" if any are missing,
-    or raising a KeyError if "raise_if_missing", or omitting the entry if
-    "omit_if_missing".
+    The ``d`` dictionary contains a ``key -> value`` mapping.
+
+    We start with a list of potential keys in ``l``, and return a list of
+    corresponding values -- substituting ``default`` if any are missing,
+    or raising :exc:`KeyError` if ``raise_if_missing`` is true, or omitting the
+    entry if ``omit_if_missing`` is true.
     """
     result = []
     for k in l:
@@ -162,7 +191,21 @@ def map_keys_to_values(l: List[Any], d: Dict[Any, Any], default: Any = None,
 def dict_diff(d1: Dict[Any, Any], d2: Dict[Any, Any],
               deleted_value: Any = None) -> Dict[Any, Any]:
     """
-    Returns a representation of the changes made to d1 to create d2.
+    Returns a representation of the changes that need to be made to ``d1`` to
+    create ``d2``.
+
+    Args:
+        d1: a dictionary
+        d2: another dictionary
+        deleted_value: value to use for deleted keys; see below
+
+    Returns:
+        dict: a dictionary of the format ``{k: v}`` where the ``k``/``v`` pairs
+        are key/value pairs that are absent from ``d1`` and present in ``d2``,
+        or present in both but with different values (in which case the ``d2``
+        value is shown). If a key ``k`` is present in ``d1`` but absent in
+        ``d2``, the result dictionary has the entry ``{k: deleted_value}``.
+
     """
     changes = {k: v for k, v in d2.items()
                if k not in d1 or d2[k] != d1[k]}
@@ -175,6 +218,17 @@ def dict_diff(d1: Dict[Any, Any], d2: Dict[Any, Any],
 def delete_keys(d: Dict[Any, Any],
                 keys_to_delete: List[Any],
                 keys_to_keep: List[Any]) -> None:
+    """
+    Deletes keys from a dictionary, in place.
+
+    Args:
+        d:
+            dictonary to modify
+        keys_to_delete:
+            if any keys are present in this list, they are deleted...
+        keys_to_keep:
+            ... unless they are present in this list.
+    """
     for k in keys_to_delete:
         if k in d and k not in keys_to_keep:
             del d[k]
@@ -183,13 +237,18 @@ def delete_keys(d: Dict[Any, Any],
 # =============================================================================
 # Lazy dictionaries
 # =============================================================================
-# https://stackoverflow.com/questions/17532929/how-to-implement-a-lazy-setdefault  # noqa
 
 class LazyDict(dict):
     """
+    A dictionary that only evaluates the argument to :func:`setdefault` or
+    :func:`get` if it needs to.
+    
+    See
+    https://stackoverflow.com/questions/17532929/how-to-implement-a-lazy-setdefault.
+    
     The ``*args``/``**kwargs`` parts are useful, but we don't want to have to
     name 'thunk' explicitly.
-    """
+    """  # noqa
     def get(self, key: Hashable, thunk: Any = None,
             *args: Any, **kwargs: Any) -> Any:
         if key in self:
@@ -211,10 +270,17 @@ class LazyDict(dict):
 
 class LazyButHonestDict(dict):
     """
+    A dictionary that provides alternatives to :func:`get` and
+    :func:`setdefault`, namely :func:`lazyget` and :func:`lazysetdefault`,
+    that only evaluate their arguments if they have to.
+
+    See
+    https://stackoverflow.com/questions/17532929/how-to-implement-a-lazy-setdefault.
+
     Compared to the StackOverflow version: no obvious need to have a default
     returning ``None``, when we're implementing this as a special function.
     In contrast, helpful to have ``*args``/``**kwargs`` options.
-    """
+    """  # noqa
     def lazyget(self, key: Hashable, thunk: Callable,
                 *args: Any, **kwargs: Any) -> Any:
         if key in self:
