@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,10 +22,11 @@
 
 ===============================================================================
 
-Support functions for Alembic, specifically to support view creation. From
-http://alembic.readthedocs.org/en/latest/cookbook.html
+**Support functions for Alembic, specifically to support view creation.**
 
-NO TYPING ANNOTATIONS - alembic uses inspect.getargspec(), which chokes
+From http://alembic.readthedocs.org/en/latest/cookbook.html.
+
+HAS NO TYPE ANNOTATIONS - Alembic uses ``inspect.getargspec()``, which chokes
 on them.
 """
 
@@ -38,6 +39,29 @@ from alembic.operations import Operations, MigrateOperation
 
 class ReplaceableObject(object):
     def __init__(self, name, sqltext):
+        """
+        Object that can be passed to the ``create_view()`` and similar
+        functions that we will register within the ``alembic.op`` namespace.
+        
+        See http://alembic.zzzcomputing.com/en/latest/cookbook.html#replaceable-objects
+
+        Args:
+            name:
+                e.g. name of a view, such as ``subject_session_view``
+
+            sqltext:
+                e.g. SQL to create the view, such as
+
+                .. code-block:: sql
+
+                    SELECT
+                        C.subject,
+                        S.*
+                    FROM
+                        config C
+                        INNER JOIN session S ON S.config_id = C.config_id
+
+        """  # noqa
         self.name = name
         self.sqltext = sqltext
 
@@ -47,19 +71,54 @@ class ReplaceableObject(object):
 # =============================================================================
 
 class ReversibleOp(MigrateOperation):
+    """
+    Represents a DDL (SQL) migration operation that can be reversed; e.g. the
+    combination of ``CREATE VIEW`` and ``DROP VIEW``.
+    """
     def __init__(self, target):
+        """
+        Args:
+            target: instance of :class:`.ReplaceableObject`
+        """
         self.target = target
 
     @classmethod
     def invoke_for_target(cls, operations, target):
+        """
+        Invokes the operation.
+
+        Args:
+            operations: instance of ``alembic.operations.base.Operations``
+            target: instance of :class:`.ReplaceableObject`
+
+        Returns:
+            result of ``alembic.operations.base.Operations.invoke``
+            performed upon an instance of this class initialized with
+            ``target``
+        """
         op = cls(target)
         return operations.invoke(op)
 
     def reverse(self):
+        """
+        Returns:
+            the ``MigrateOperation`` representing the reverse of this operation
+        """
         raise NotImplementedError()
 
     @classmethod
     def _get_object_from_version(cls, operations, ident):
+        """
+        Returns a Python object from an Alembic migration module (script).
+
+        Args:
+            operations: instance of ``alembic.operations.base.Operations``
+            ident: string of the format ``version.objname``
+
+        Returns:
+            the object whose name is ``objname`` within the Alembic migration
+            script identified by ``version``
+        """
         version, objname = ident.split(".")
 
         module_ = operations.get_context().script.get_revision(version).module
@@ -91,12 +150,18 @@ class ReversibleOp(MigrateOperation):
 @Operations.register_operation("create_view", "invoke_for_target")
 @Operations.register_operation("replace_view", "replace")
 class CreateViewOp(ReversibleOp):
+    """
+    Represents ``CREATE VIEW`` (reversed by ``DROP VIEW``).
+    """
     def reverse(self):
         return DropViewOp(self.target)
 
 
 @Operations.register_operation("drop_view", "invoke_for_target")
 class DropViewOp(ReversibleOp):
+    """
+    Represents ``DROP VIEW`` (reversed by ``CREATE VIEW``).
+    """
     def reverse(self):
         return CreateViewOp(self.view)
 
@@ -104,12 +169,20 @@ class DropViewOp(ReversibleOp):
 @Operations.register_operation("create_sp", "invoke_for_target")
 @Operations.register_operation("replace_sp", "replace")
 class CreateSPOp(ReversibleOp):
+    """
+    Represents ``CREATE FUNCTION`` (reversed by ``DROP FUNCTION``)
+    [sp = stored procedure].
+    """
     def reverse(self):
         return DropSPOp(self.target)
 
 
 @Operations.register_operation("drop_sp", "invoke_for_target")
 class DropSPOp(ReversibleOp):
+    """
+    Represents ``DROP FUNCTION`` (reversed by ``CREATE FUNCTION``)
+    [sp = stored procedure].
+    """
     def reverse(self):
         return CreateSPOp(self.target)
 
@@ -120,6 +193,16 @@ class DropSPOp(ReversibleOp):
 
 @Operations.implementation_for(CreateViewOp)
 def create_view(operations, operation):
+    """
+    Implements ``CREATE VIEW``.
+
+    Args:
+        operations: instance of ``alembic.operations.base.Operations``
+        operation: instance of :class:`.ReversibleOp`
+
+    Returns:
+        ``None``
+    """
     operations.execute("CREATE VIEW %s AS %s" % (
         operation.target.name,
         operation.target.sqltext
@@ -128,11 +211,31 @@ def create_view(operations, operation):
 
 @Operations.implementation_for(DropViewOp)
 def drop_view(operations, operation):
+    """
+    Implements ``DROP VIEW``.
+
+    Args:
+        operations: instance of ``alembic.operations.base.Operations``
+        operation: instance of :class:`.ReversibleOp`
+
+    Returns:
+        ``None``
+    """
     operations.execute("DROP VIEW %s" % operation.target.name)
 
 
 @Operations.implementation_for(CreateSPOp)
 def create_sp(operations, operation):
+    """
+    Implements ``CREATE FUNCTION``.
+
+    Args:
+        operations: instance of ``alembic.operations.base.Operations``
+        operation: instance of :class:`.ReversibleOp`
+
+    Returns:
+        ``None``
+    """
     operations.execute(
         "CREATE FUNCTION %s %s" % (
             operation.target.name, operation.target.sqltext
@@ -142,4 +245,14 @@ def create_sp(operations, operation):
 
 @Operations.implementation_for(DropSPOp)
 def drop_sp(operations, operation):
+    """
+    Implements ``DROP FUNCTION``.
+
+    Args:
+        operations: instance of ``alembic.operations.base.Operations``
+        operation: instance of :class:`.ReversibleOp`
+
+    Returns:
+        ``None``
+    """
     operations.execute("DROP FUNCTION %s" % operation.target.name)
