@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,11 +22,12 @@
 
 ===============================================================================
 
-Miscellany to help with WSGI work.
+**Miscellany to help with WSGI work.**
 
 """
 
-from typing import Callable, Dict, Iterable, List, Tuple
+from types import TracebackType
+from typing import Callable, Dict, Iterable, List, Optional, Tuple, Type
 
 # =============================================================================
 # Type hints for WSGI
@@ -37,11 +38,24 @@ TYPE_WSGI_ENVIRON = Dict[str, str]
 TYPE_WSGI_STATUS = str
 TYPE_WSGI_RESPONSE_HEADERS = List[Tuple[str, str]]
 TYPE_WSGI_START_RESP_RESULT = Callable[[str], None]  # call with e.g. write(body_data)  # noqa
-TYPE_WSGI_START_RESPONSE = Callable[[TYPE_WSGI_STATUS,
-                                     TYPE_WSGI_RESPONSE_HEADERS],
-                                    TYPE_WSGI_START_RESP_RESULT]
-# There is an optional third parameter to start_response()
-TYPE_WSGI_APP_RESULT = Iterable[str]
+TYPE_WSGI_EXC_INFO = Tuple[
+    # https://docs.python.org/3/library/sys.html#sys.exc_info
+    Optional[Type[BaseException]],  # type
+    Optional[BaseException],  # value
+    Optional[TracebackType]  # traceback
+]
+TYPE_WSGI_START_RESPONSE = Callable[
+    # There is an optional third parameter to start_response():
+    [TYPE_WSGI_STATUS,  # status
+     TYPE_WSGI_RESPONSE_HEADERS,  # headers
+     Optional[TYPE_WSGI_EXC_INFO]],  # exc_info
+    TYPE_WSGI_START_RESP_RESULT
+]
+TYPE_WSGI_APP_RESULT = Iterable[bytes]
+# ... must be BYTE STRINGS, not str; see
+# https://www.python.org/dev/peps/pep-0333/#unicode-issues
+# and also
+# https://github.com/fgallaire/wsgiserver/issues/2
 TYPE_WSGI_APP = Callable[[TYPE_WSGI_ENVIRON, TYPE_WSGI_START_RESPONSE],
                          TYPE_WSGI_APP_RESULT]
 
